@@ -1,9 +1,10 @@
 package HTML::FormFu::Constraint::Callback;
 
-use strict;
-use base 'HTML::FormFu::Constraint';
+use Moose;
+use MooseX::Attribute::Chained;
+extends 'HTML::FormFu::Constraint';
 
-__PACKAGE__->mk_item_accessors(qw( callback ));
+has callback => ( is => 'rw', traits  => ['Chained'] );
 
 sub constrain_value {
     my ( $self, $value, $params ) = @_;
@@ -17,6 +18,8 @@ sub constrain_value {
     return $ok;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
