@@ -1,34 +1,36 @@
 package HTML::FormFu::Element::Date;
+use Moose;
+use MooseX::Attribute::Chained;
 
-use strict;
-use base 'HTML::FormFu::Element::Multi';
-use MRO::Compat;
-use mro 'c3';
+extends 'HTML::FormFu::Element::Multi';
 
 use HTML::FormFu::Util qw( _filter_components _parse_args );
 use DateTime;
 use DateTime::Format::Builder;
 use DateTime::Format::Natural;
 use DateTime::Locale;
+use Moose::Util qw( apply_all_roles );
 use Scalar::Util qw( blessed );
 use List::MoreUtils qw( all none uniq );
 use Carp qw( croak );
 
 __PACKAGE__->mk_attrs(qw( day  month  year ));
 
-__PACKAGE__->mk_accessors( qw(
-        _known_fields
-        printf_day
-        printf_month
-        printf_year
-) );
+has auto_inflate          => ( is => 'rw', traits => ['Chained'] );
+has default_natural       => ( is => 'rw', traits => ['Chained'] );
+has default_datetime_args => ( is => 'rw', traits => ['Chained'] );
+has printf_day            => ( is => 'rw', traits => ['Chained'] );
+has printf_month          => ( is => 'rw', traits => ['Chained'] );
+has printf_year           => ( is => 'rw', traits => ['Chained'] );
 
-__PACKAGE__->mk_item_accessors( qw(
-        strftime
-        auto_inflate
-        default_natural
-        default_datetime_args
-) );
+has _known_fields => ( is => 'rw' );
+
+has strftime => (
+    is      => 'rw',
+    default => '%d-%m-%Y',
+    lazy    => 1,
+    traits  => ['Chained'],
+);
 
 *default = \&value;
 
@@ -58,39 +60,34 @@ for my $method ( qw(
     *{$name} = $sub;
 }
 
-sub new {
-    my $self = shift->next::method(@_);
+after BUILD => sub {
+    my ( $self, $args ) = @_;
 
-    $self->strftime("%d-%m-%Y");
+    $self->printf_day(   '%d' );
+    $self->printf_month( '%d' );
+    $self->printf_year(  '%d' );
 
-    $self->_known_fields( [qw/ day month year /] );
+    $self->_known_fields( [qw( day month year )] );
 
-    $self->printf_day  ('%d');
-    $self->printf_month('%d');
-    $self->printf_year ('%d');
-
-    $self->field_order( [qw/ day month year /] );
+    $self->field_order( [qw( day month year )] );
 
     $self->day( {
-            type   => '_MultiSelect',
             prefix => [],
         } );
 
     $self->month( {
-            type   => '_MultiSelect',
             prefix => [],
         } );
 
     $self->year( {
-            type    => '_MultiSelect',
-            prefix  => [],
-            less    => 0,
-            plus    => 10,
+            prefix => [],
+            less   => 0,
+            plus   => 10,
             reverse => 0,
         } );
 
-    return $self;
-}
+    return;
+};
 
 sub value {
     my ( $self, $value ) = @_;
@@ -152,11 +149,7 @@ sub _date_defaults {
 
     my $default;
 
-    if ( defined( $default = $self->default_natural ) ) {
-        my $parser = DateTime::Format::Natural->new;
-        $default = $parser->parse_datetime($default);
-    }
-    elsif ( defined( $default = $self->default ) && length $default ) {
+    if ( defined( $default = $self->default ) && length $default ) {
         
         if ( !$self->form->submitted || $self->render_processed_value ) {
             for my $deflator ( @{ $self->_deflators } ) {
@@ -172,7 +165,25 @@ sub _date_defaults {
 
             $default = $builder->parse_datetime($default);
         }
-    } else {
+    }
+    elsif ( defined( $default = $self->default_natural ) ) {
+        my $parser;
+        
+        if ( defined( my $datetime_args = $self->default_datetime_args ) ) {
+            if ( exists $datetime_args->{set_time_zone} ) {
+                my $tz = $datetime_args->{set_time_zone};
+                $parser = DateTime::Format::Natural->new( time_zone => $tz );
+            }
+            else {
+                $parser = DateTime::Format::Natural->new;
+            }
+        }
+        else {
+            $parser = DateTime::Format::Natural->new;
+        }
+        $default = $parser->parse_datetime( $default );
+    }
+    else {
       $default = undef;
     }
 
@@ -204,16 +215,26 @@ sub _add_day {
         ? @{ $day->{prefix} }
         : $day->{prefix};
 
+    if (exists $day->{prefix_loc}) {
+        @day_prefix
+            = ref $day->{prefix_loc}
+            ? map { $self->form->localize($_) } @{ $day->{prefix_loc} }
+            : $self->form->localize($day->{prefix_loc});
+    }
+
     @day_prefix = map { [ '', $_ ] } @day_prefix;
 
-    $self->element( {
-            type    => $day->{type},
-            name    => $day_name,
-            options => [ @day_prefix, map { [ $_, $_ ] } 1 .. 31 ],
+    my $element = $self->element( {
+            type       => 'Select',
+            name       => $day_name,
+            options    => [ @day_prefix, map { [ $_, $_ ] } 1 .. 31 ],
+            attributes => $day->{attributes},
 
             defined $day->{default} ? ( default => $day->{default} ) : (),
         } );
 
+    apply_all_roles( $element, 'HTML::FormFu::Role::Element::MultiElement' );
+
     return;
 }
 
@@ -231,18 +252,28 @@ sub _add_month {
         ? @{ $month->{prefix} }
         : $month->{prefix};
 
+    if (exists $month->{prefix_loc}) {
+        @month_prefix
+            = ref $month->{prefix_loc}
+            ? map { $self->form->localize($_) } @{ $month->{prefix_loc} }
+            : $self->form->localize($month->{prefix_loc});
+    }
+
     @month_prefix = map { [ '', $_ ] } @month_prefix;
 
     my $options = [ @month_prefix, map { [ $_ + 1, $months[$_] ] } 0 .. 11 ];
 
-    $self->element( {
-            type    => $month->{type},
-            name    => $month_name,
-            options => $options,
+    my $element = $self->element( {
+            type       => 'Select',
+            name       => $month_name,
+            options    => $options,
+            attributes => $month->{attributes},
 
             defined $month->{default} ? ( default => $month->{default} ) : (),
         } );
 
+    apply_all_roles( $element, 'HTML::FormFu::Role::Element::MultiElement' );
+
     return;
 }
 
@@ -272,16 +303,26 @@ sub _add_year {
         ? @{ $year->{prefix} }
         : $year->{prefix};
 
+    if (exists $year->{prefix_loc}) {
+        @year_prefix
+            = ref $year->{prefix_loc}
+            ? map { $self->form->localize($_) } @{ $year->{prefix_loc} }
+            : $self->form->localize($year->{prefix_loc});
+    }
+
     @year_prefix = map { [ '', $_ ] } @year_prefix;
 
-    $self->element( {
-            type    => $year->{type},
-            name    => $year_name,
-            options => [ @year_prefix, map { [ $_, $_ ] } @years ],
+    my $element = $self->element( {
+            type       => 'Select',
+            name       => $year_name,
+            options    => [ @year_prefix, map { [ $_, $_ ] } @years ],
+            attributes => $year->{attributes},
 
             defined $year->{default} ? ( default => $year->{default} ) : (),
         } );
 
+    apply_all_roles( $element, 'HTML::FormFu::Role::Element::MultiElement' );
+
     return;
 }
 
@@ -320,6 +361,20 @@ sub _build_month_list {
     return @months;
 }
 
+sub _build_number_list {
+    my ( $self, $start, $end, $interval ) = @_;
+    
+    $interval ||= 1;
+    
+    my @list;
+    
+    for ( my $i = $start; $i <= $end; $i += $interval ) {
+        push @list, $i;
+    }
+    
+    return @list;
+}
+
 sub _build_name {
     my ( $self, $type ) = @_;
 
@@ -373,7 +428,7 @@ sub process {
 
     $self->_add_elements;
 
-    return $self->next::method(@args);
+    return $self->SUPER::process(@args);
 }
 
 sub process_input {
@@ -412,7 +467,7 @@ sub process_input {
         $self->set_nested_hash_value( $input, $self->nested_name, $value );
     }
 
-    return $self->next::method($input);
+    return $self->SUPER::process_input($input);
 }
 
 sub render_data {
@@ -422,7 +477,7 @@ sub render_data {
 sub render_data_non_recursive {
     my ( $self, $args ) = @_;
 
-    my $render = $self->next::method( {
+    my $render = $self->SUPER::render_data_non_recursive( {
             elements => [ map { $_->render_data } @{ $self->_elements } ],
             $args ? %$args : (),
         } );
@@ -430,6 +485,8 @@ sub render_data_non_recursive {
     return $render;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -540,6 +597,20 @@ menu.
 Each value is only used as the label for a select item - the value for each 
 of these items is always the empty string C<''>.
 
+=head3 prefix_loc
+
+Arguments: $localization_key
+
+Arguments: \@localization_keys
+
+A localized string or arrayref of localized strings to be inserted into the
+start of the select menu.
+
+Each value is localized and then only used as the label for a select item
+- the value for each of these items is always the empty string C<''>.
+
+Use C<prefix_loc> insted of C<prefix>.
+
 =head2 month
 
 Arguments: \%setting
@@ -566,6 +637,20 @@ menu.
 Each value is only used as the label for a select item - the value for each 
 of these items is always the empty string C<''>.
 
+=head3 prefix_loc
+
+Arguments: $localization_key
+
+Arguments: \@localization_keys
+
+A localized string or arrayref of localized strings to be inserted into the
+start of the select menu.
+
+Each value is localized and then only used as the label for a select item
+- the value for each of these items is always the empty string C<''>.
+
+Use C<prefix_loc> insted of C<prefix>.
+
 =head3 names
 
 Arguments: \@months
@@ -609,6 +694,20 @@ menu.
 Each value is only used as the label for a select item - the value for each 
 of these items is always the empty string C<''>.
 
+=head3 prefix_loc
+
+Arguments: $localization_key
+
+Arguments: \@localization_keys
+
+A localized string or arrayref of localized strings to be inserted into the
+start of the select menu.
+
+Each value is localized and then only used as the label for a select item
+- the value for each of these items is always the empty string C<''>.
+
+Use C<prefix_loc> insted of C<prefix>.
+
 =head3 list
 
 Arguments: \@years
