@@ -8,7 +8,7 @@ with 'HTML::FormFu::Role::Element::Field',
 
 use HTML::FormFu::Util qw( process_attrs );
 
-__PACKAGE__->mk_attr_accessors(qw( cols rows ));
+__PACKAGE__->mk_attr_accessors(qw( cols rows placeholder ));
 
 after BUILD => sub {
     my $self = shift;
@@ -51,10 +51,17 @@ sub _string_field {
 
     # textarea_tag template
 
-    my $html = sprintf qq{<textarea name="%s"%s>},
-        $render->{nested_name},
-        process_attrs( $render->{attributes} ),
-        ;
+    my $html .= "<textarea";
+
+    $html .= sprintf qq{ name="%s"}, $render->{nested_name};
+
+    if ( defined $self->placeholder ) {
+        $html .= sprintf qq{ placeholder="%s"}, $self->placeholder;
+    }
+
+    $html .= process_attrs( $render->{attributes} );
+
+    $html .= '>';
 
     if ( defined $render->{value} ) {
         $html .= $render->{value};
@@ -93,6 +100,23 @@ Sets the C<textarea> tag's C<cols> attribute.
 
 Sets the C<textarea> tag's C<rows> attribute.
 
+=head2 placeholder
+
+Sets the HTML5 attribute C<placeholder> to the specified value.
+
+=head2 placeholder_xml
+
+If you don't want the C<placeholder> attribute to be XML-escaped, use the L</placeholder_xml> 
+method instead of L</placeholder>.
+
+Arguments: $string
+
+=head2 placeholder_loc
+
+Arguments: $localization_key
+
+Set the  C<placeholder> attribute using a L10N key.
+
 =head1 SEE ALSO
 
 Is a sub-class of, and inherits methods from 
